﻿// Kerbal Engineer Redux
// Author:  CYBUTEK
// License: Attribution-NonCommercial-ShareAlike 3.0 Unported

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class FlightEngineer : EngineerModule
    {
        public Vessel activeVessel;
        public Rendevous rendevous;
        bool windowDraggable = true;
        bool engineerStarted = true;
        bool showOrbital = false;
        bool showSurface = false;
        bool showVessel = false;
        bool showRendevous = false;

        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            StartEngineer();

            Debug("FlightEngineer: OnStart(" + state + ")");
        }

        public override void OnUpdate()
        {
            
            base.OnUpdate();
            if (!engineerStarted)
                StartEngineer();

            //Debug("FlightEngineer: OnUpdate");
        }

        public void StartEngineer()
        {
            if (state != StartState.Editor && this.vessel.rootPart == FlightGlobals.ActiveVessel.rootPart)
            {
                activeVessel = this.vessel;
                IsPrimary = CheckIsPrimary("FlightEngineer", activeVessel.parts);

                if (IsPrimary)
                {
                    LoadSettings();

                    if (winPos.x == 0 && winPos.y == 0)
                        winPos = new Rect(UnityEngine.Screen.width - 275, 0, 0, 0);

                    RenderingManager.AddToPostDrawQueue(0, DrawGUI);
                    rendevous = new Rendevous(this);
                    this.part.force_activate();
                    engineerStarted = true;
                    return;
                }
            }

            engineerStarted = false;
        }

        public override void DrawGUI()
        {
            base.DrawGUI();

            if (this.part == null || activeVessel != FlightGlobals.ActiveVessel)
            {
                RenderingManager.RemoveFromPostDrawQueue(0, DrawGUI);
                engineerStarted = false;

                return;
            }

            ship.Update(activeVessel.parts);
            Rect winPosBefore = winPos;
            winPos = GUILayout.Window(3, winPos, MainWindow, "Flight Engineer", winStyle);
            if (winPos != winPosBefore) SaveSettings();
        }

        protected override void InitGUI()
        {
            base.InitGUI();
            winStyle.fixedWidth = 275;
            dataStyle.alignment = TextAnchor.UpperRight;
        }

        public void MainWindow(int windowId)
        {
            DrawOptions();

            if (showOrbital)
                DrawOrbital();

            if (showSurface)
                DrawSurface();

            if (showVessel)
                DrawVessel();

            if (showRendevous)
                DrawRendevous();

            if (windowDraggable) GUI.DragWindow(new Rect(0, 0, winPos.width, winPos.height));
        }

        private void DrawOptions()
        {
            GUILayout.BeginHorizontal();
            GUILayout.BeginVertical();
            if (showOrbital)
            {
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }
            if (GUILayout.Button("ORB", buttonStyle))
            {
                showOrbital = !showOrbital;
                winPos.height = 0;
            }
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            if (showSurface)
            {
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }
            if (GUILayout.Button("SUR", buttonStyle))
            {
                showSurface = !showSurface;
                winPos.height = 0;
            }
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            if (showVessel)
            {
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }
            if (GUILayout.Button("VES", buttonStyle))
            {
                showVessel = !showVessel;
                winPos.height = 0;
            }
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            if (showRendevous)
            {
                buttonStyle.normal = HighLogic.Skin.button.active;
            }
            else
            {
                buttonStyle.normal = HighLogic.Skin.button.normal;
            }
            if (GUILayout.Button("RDV", buttonStyle))
            {
                showRendevous = !showRendevous;
                winPos.height = 0;
            }
            buttonStyle.normal = HighLogic.Skin.button.normal;
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawOrbital()
        {
            GUILayout.Label("Orbital Display", titleStyle);

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Apoapsis Height", headStyle);
            GUILayout.Label("Periapsis Height", headStyle);
            GUILayout.Label("Time to Apoapsis", headStyle);
            GUILayout.Label("Time to Periapsis", headStyle);
            GUILayout.Label("Inclination", headStyle);
            GUILayout.Label("Eccentricity", headStyle);
            GUILayout.Label("Orbital Period", headStyle);
            GUILayout.Label("Semi-major Axis", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatDistance(activeVessel.orbit.ApA), dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.orbit.PeA), dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToAp.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(activeVessel.orbit.timeToPe.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(activeVessel.orbit.inclination.ToString("0.000") + "°", dataStyle);
            GUILayout.Label(activeVessel.orbit.eccentricity.ToString("0.000000"), dataStyle);
            GUILayout.Label(activeVessel.orbit.period.ToString("0.000") + "s", dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.orbit.semiMajorAxis), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawSurface()
        {
            GUILayout.Label("Surface Display", titleStyle);

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Altitude (Sea Level)", headStyle);
            GUILayout.Label("Altitude (Terrain)", headStyle);
            GUILayout.Label("Vertical Speed", headStyle);
            GUILayout.Label("Horizontal Speed", headStyle);
            GUILayout.Label("Longitude", headStyle);
            GUILayout.Label("Latitude", headStyle);
            GUILayout.Label("G-Force", headStyle);
            GUILayout.Label("Atmospheric Density", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(FormatDistance(activeVessel.mainBody.GetAltitude(activeVessel.CoM)), dataStyle);
            GUILayout.Label(FormatDistance(activeVessel.mainBody.GetAltitude(activeVessel.CoM) - activeVessel.terrainAltitude), dataStyle);
            GUILayout.Label(FormatSpeed(activeVessel.verticalSpeed), dataStyle);
            GUILayout.Label(FormatSpeed(activeVessel.horizontalSrfSpeed), dataStyle);
            GUILayout.Label(activeVessel.longitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.latitude.ToString("0.000000") + "°", dataStyle);
            GUILayout.Label(activeVessel.geeForce.ToString("0.000"), dataStyle);
            GUILayout.Label(activeVessel.atmDensity.ToString("0.000000"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        } 

        private void DrawVessel()
        {
            GUILayout.Label("Vessel Display", titleStyle);

            GUILayout.BeginHorizontal(areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Delta-V (Stage)", headStyle);
            GUILayout.Label("Delta-V (Total)", headStyle);
            GUILayout.Label("Fuel", headStyle);
            GUILayout.Label("Mass", headStyle);
            GUILayout.Label("Thrust", headStyle);
            GUILayout.Label("TWR", headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(GetStage(activeVessel.currentStage).DeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).TotalDeltaV.ToString("0") + " m/sec", dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Fuel.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalFuel.ToString("0.00"), dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Mass.ToString("0.00") + " / " + GetStage(activeVessel.currentStage).TotalMass.ToString("0.00"), dataStyle);
            GUILayout.Label(GetStage(activeVessel.currentStage).Thrust.ToString("0") + " kN", dataStyle);
            GUILayout.Label((GetStage(activeVessel.currentStage).Thrust / (GetStage(activeVessel.currentStage).TotalMass * (activeVessel.mainBody.gravParameter / Math.Pow(activeVessel.mainBody.Radius, 2)))).ToString("0.00"), dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawRendevous()
        {
            rendevous.Draw();
        }

        public string FormatSpeed(double number, bool useNotation = true)
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }

            string notation = "m/s";
            string strNumber = number.ToString("#,##0.0");

            if (number < 1)
            {
                number *= 100;
                if (number >= 1)
                {
                    notation = "cm/s";
                    strNumber = number.ToString("0.00");
                }

                if (number < 1)
                {
                    notation = "mm/s";
                    number *= 100;
                    strNumber = number.ToString("0");
                }
            }

            if (number >= 1000)
            {
                notation = "km/s";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            if (!useNotation) notation = "";
            return strNumber + notation;
        }

        public string FormatDistance(double number)
        {
            bool negative = false;
            if (number < 0)
            {
                number = -number;
                negative = true;
            }


            string notation = "m";
            string strNumber = number.ToString("#,##0");

            if (number < 100)
                strNumber = number.ToString("0.000");

            if (number >= 10000)
            {
                notation = "km";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (number >= 10000)
            {
                notation = "Mm";
                number /= 1000;
                strNumber = number.ToString("0.000");
            }

            if (negative)
                strNumber = "-" + strNumber;

            return strNumber + notation;
        }

        private void SaveSettings()
        {
            settings.SetValue("FlightEngineer::winPos.x", winPos.x.ToString());
            settings.SetValue("FlightEngineer::winPos.y", winPos.y.ToString());
            settings.Save();
        }

        private void LoadSettings()
        {
            settings.Load();
            winPos.x = Convert.ToSingle(settings.GetValue("FlightEngineer::winPos.x"));
            winPos.y = Convert.ToSingle(settings.GetValue("FlightEngineer::winPos.y"));
        }
    }
}
