﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Engineer
{
    class Rendevous
    {
        enum TargetType { None, Vessel, Debris, Cellestial };

        FlightEngineer fe;
        Vessel targetVes;
        CelestialBody targetBody;
        Vector2 scrollPosition = Vector2.zero;
        bool hasTarget = false;
        TargetType typeOfTarget = TargetType.None;

        public Rendevous(FlightEngineer flightEngineer)
        {
            fe = flightEngineer;
        }

        public void Draw()
        {
            if (!hasTarget)
            {
                switch (typeOfTarget)
                {
                    case TargetType.Vessel:
                        DrawTargets();
                        break;
                    case TargetType.Cellestial:
                        DrawTargets();
                        break;
                    case TargetType.Debris:
                        DrawTargets();
                        break;
                    default:
                        DrawTargetTypes();
                        break;
                }
            }
            else
            {
                DrawDetails();
            }
        }

        private void DrawTargetTypes()
        {
            GUILayout.Label("Rendezvous Display", fe.titleStyle);
            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            if (GUILayout.Button("Vessels", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Vessel;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Debris", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Debris;
                fe.winPos.height = 0;
            }
            if (GUILayout.Button("Cellestial Bodies", fe.buttonStyle))
            {
                typeOfTarget = TargetType.Cellestial;
                fe.winPos.height = 0;
            }
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();
        }

        private void DrawTargets()
        {
            

            if (typeOfTarget == TargetType.Cellestial)
            {
                GUILayout.Label("Cellestial Bodies", fe.titleStyle);
                GUILayout.BeginHorizontal(fe.areaStyle);
                GUILayout.BeginVertical();
                foreach (CelestialBody b in FlightGlobals.Bodies)
                {
                    if (b == fe.activeVessel.orbit.referenceBody) continue;
                    if (b.name == "Sun") continue;

                    if (GUILayout.Button(b.name, fe.buttonStyle))
                    {
                        targetBody = b;
                        hasTarget = true;
                        fe.winPos.height = 0;
                        return;
                    }
                    }
            }
            else if (typeOfTarget != TargetType.None)
            {
                if (typeOfTarget == TargetType.Debris)
                    GUILayout.Label("Debris", fe.titleStyle);

                if (typeOfTarget == TargetType.Vessel)
                    GUILayout.Label("Vessels", fe.titleStyle);

                GUILayout.BeginHorizontal(fe.areaStyle);
                GUILayout.BeginVertical();
                foreach (Vessel v in FlightGlobals.Vessels)
                {
                    if (v == fe.activeVessel) continue;
                    if (v.LandedOrSplashed) continue;

                    if (typeOfTarget == TargetType.Debris)
                        if (v.isCommandable) continue;

                    if (typeOfTarget == TargetType.Vessel)
                        if (!v.isCommandable) continue;

                    if (v.mainBody == fe.activeVessel.mainBody)
                    {
                        if (GUILayout.Button(v.vesselName, fe.buttonStyle))
                        {
                            targetVes = v;
                            hasTarget = true;
                            fe.winPos.height = 0;
                            return;
                        }
                    }
                }

            }

            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            if (GUILayout.Button("Back to Target Type", fe.buttonStyle))
            {
                targetVes = null;
                targetBody = null;
                hasTarget = false;
                typeOfTarget = TargetType.None;
                fe.winPos.height = 0;
            }
        }

        private void DrawDetails()
        {
            Vector3d active = Vector3d.zero;
            Vector3d target = Vector3d.zero;
            string name = null;
            double phaseAngle = 0.0f;
            double distance = 0.0f;
            double relInclination = 0.0f;
            double altitude = 0.0f;
            double velocityX = 0.0f;
            double velocityY = 0.0f;
            double velocityZ = 0.0f;
            double relVelocity = 0.0f;

            if (typeOfTarget == TargetType.Vessel || typeOfTarget == TargetType.Debris)
            {
                name = targetVes.vesselName;
                phaseAngle = Vector3d.Angle(fe.activeVessel.orbit.referenceBody.position - fe.activeVessel.CoM, targetVes.orbit.referenceBody.position - targetVes.CoM);
                relInclination = targetVes.orbit.inclination - fe.activeVessel.orbit.inclination;
                altitude = targetVes.orbit.altitude;
                velocityX = targetVes.obt_velocity.x;
                velocityY = targetVes.obt_velocity.y;
                velocityZ = targetVes.obt_velocity.z;
                relVelocity = Vector3d.Cross(targetVes.obt_velocity, fe.activeVessel.obt_velocity).magnitude;
                distance = Vector3d.Distance(fe.activeVessel.CoM, targetVes.CoM);
            }
            else if (typeOfTarget == TargetType.Cellestial)
            {
                name = targetBody.name;
                phaseAngle = Vector3d.Angle(fe.activeVessel.orbit.referenceBody.position - fe.activeVessel.CoM, targetBody.position - fe.activeVessel.CoM);
                relInclination = targetBody.orbit.inclination - fe.activeVessel.orbit.inclination;
                altitude = targetBody.orbit.altitude;
                distance = Vector3d.Distance(fe.activeVessel.CoM, targetBody.position);
            }

            GUILayout.Label(name, fe.titleStyle);

            GUILayout.BeginHorizontal(fe.areaStyle);
            GUILayout.BeginVertical();
            GUILayout.Label("Phase Angle", fe.headStyle);
            GUILayout.Label("Relative Inclination", fe.headStyle);
            GUILayout.Label("Altitude", fe.headStyle);
            GUILayout.Label("Distance", fe.headStyle);
            GUILayout.Label("Forward Velocity", fe.headStyle);
            GUILayout.Label("Horizontal Velocity", fe.headStyle);
            GUILayout.Label("Vertical Velocity", fe.headStyle);
            GUILayout.EndVertical();

            GUILayout.BeginVertical();
            GUILayout.Label(phaseAngle.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(relInclination.ToString("0.000") + "°", fe.dataStyle);
            GUILayout.Label(fe.FormatDistance(altitude), fe.dataStyle);
            GUILayout.Label(fe.FormatDistance(distance), fe.dataStyle);
            GUILayout.Label(fe.FormatSpeed(velocityX - fe.activeVessel.obt_velocity.x), fe.dataStyle);
            GUILayout.Label(fe.FormatSpeed(velocityY - fe.activeVessel.obt_velocity.y), fe.dataStyle);
            GUILayout.Label(fe.FormatSpeed(velocityZ - fe.activeVessel.obt_velocity.z), fe.dataStyle);
            GUILayout.EndVertical();
            GUILayout.EndHorizontal();

            if (GUILayout.Button("Back to Targets", fe.buttonStyle))
            {
                targetVes = null;
                targetBody = null;
                hasTarget = false;
                fe.winPos.height = 0;
            }
        }
    }
}
